@@ -10,9 +10,9 @@
 *
 * $Project: /Convert-Binary-C $
 * $Author: mhx $
-* $Date: 2003/06/06 13:41:29 +0200 $
-* $Revision: 18 $
-* $Snapshot: /Convert-Binary-C/0.48 $
+* $Date: 2003/11/21 13:46:32 +0100 $
+* $Revision: 19 $
+* $Snapshot: /Convert-Binary-C/0.49 $
 * $Source: /ctlib/ctparse.h $
 *
 ********************************************************************************
@@ -126,7 +126,7 @@ typedef struct {
   HashTable  htStructs;
   HashTable  htTypedefs;
   HashTable  htFiles;
-  char      *errstr;
+  LinkedList errorStack;
 } CParseInfo;
 
 typedef enum {
@@ -164,7 +164,4 @@ ErrorGTI get_type_info( const CParseConfig *pCPC, TypeSpec *pTS, Declarator *pDe
                         unsigned *pSize, unsigned *pAlign, unsigned *pItemSize,
                         u_32 *pFlags );
 
-#define format_error CTlib_format_error
-void format_error( CParseInfo *pCPI, char *format, ... );
-
 #endif
